@@ -21,14 +21,17 @@ extern void stirling(mpz_t r, unsigned long n, unsigned long m, UV type);
 extern void binomial(mpz_t r, UV n, UV k);
 extern void partitions(mpz_t npart, UV n);
 extern void factorialmod(mpz_t r, UV n, mpz_t m);
+extern void multifactorial(mpz_t r, UV n, UV k);
+extern void factorial_sum(mpz_t r, UV n);
+extern void subfactorial(mpz_t r, UV n);
+extern void rising_factorial(mpz_t r, UV x, UV n);
+extern void falling_factorial(mpz_t r, UV x, UV n);
 
-extern UV   is_power(mpz_t n, UV a);
 extern int  is_carmichael(mpz_t n);
 extern int  is_fundamental(mpz_t n);
 extern int  is_totient(mpz_t n);
 extern void polygonal_nth(mpz_t r, mpz_t n, UV k);
 
-extern UV   prime_power(mpz_t prime, mpz_t n);
 extern void exp_mangoldt(mpz_t res, mpz_t n);
 
 extern uint32_t* partial_sieve(mpz_t start, UV length, UV maxprime);
