@@ -142,13 +142,54 @@ void mpz_isaac_urandomm(mpz_t rop, mpz_t n)
     do {
       mpz_isaac_urandomb(rop, nbits+8);
     } while (mpz_cmp(rop, rmax) >= 0 && count-- > 0);
+    mpz_clear(rmax);
     mpz_mod(rop, rop, n);
   }
 }
 
+/* a=0, return power.  a>1, return bool if an a-th power */
+UV is_power(mpz_t n, UV a)
+{
+  if (mpz_cmp_ui(n,3) <= 0 && a == 0)
+    return 0;
+  else if (a == 1)
+    return 1;
+  else if (a == 2)
+    return mpz_perfect_square_p(n);
+  else {
+    UV result;
+    mpz_t t;
+    mpz_init(t);
+    result = (a == 0)  ?  power_factor(n, t)  :  (UV)mpz_root(t, n, a);
+    mpz_clear(t);
+    return result;
+  }
+}
+
+UV prime_power(mpz_t prime, mpz_t n)
+{
+  UV k;
+  if (mpz_even_p(n)) {
+    k = mpz_scan1(n, 0);
+    if (k+1 == mpz_sizeinbase(n, 2)) {
+      mpz_set_ui(prime, 2);
+      return k;
+    }
+    return 0;
+  }
+  if (_GMP_is_prob_prime(n)) {
+    mpz_set(prime, n);
+    return 1;
+  }
+  k = power_factor(n, prime);
+  if (k && !_GMP_is_prob_prime(prime))
+    k = 0;
+  return k;
+}
+
 int is_primitive_root(mpz_t ina, mpz_t n, int nprime)
 {
-  mpz_t a, s, sreduced, t, *factors;
+  mpz_t a, s, r, sreduced, t, *factors;
   int ret, i, nfactors, *exponents;
 
   if (mpz_sgn(n) == 0)
@@ -157,6 +198,10 @@ int is_primitive_root(mpz_t ina, mpz_t n, int nprime)
     mpz_neg(n,n);
   if (mpz_cmp_ui(n,1) == 0)
     return 1;
+  if (mpz_cmp_ui(n,4) <= 0)
+    return mpz_get_ui(ina) == mpz_get_ui(n)-1;
+  if (mpz_divisible_2exp_p(n,2))
+    return 0;
 
   mpz_init(a);
   mpz_mod(a,ina,n);
@@ -168,14 +213,33 @@ int is_primitive_root(mpz_t ina, mpz_t n, int nprime)
     mpz_clear(a);
     return 0;
   }
+
+  mpz_init(t);
   if (nprime) {
     mpz_sub_ui(s, n, 1);
-  } else {
-    totient(s, n);
+  } else { /* totient(s, n); */   /* Fine, but slow. */
+    UV k;
+    mpz_init(r);
+    if (mpz_odd_p(n)) mpz_set(t, n); else mpz_fdiv_q_2exp(t, n, 1);
+    k = prime_power(r, t);
+    if (!k) {  /* Not of form p^a or 2p^a */
+      mpz_clear(r); mpz_clear(t); mpz_clear(s); mpz_clear(a); return 0;
+    }
+    mpz_divexact(t, t, r);
+    mpz_mul(s, t, r);  mpz_sub(s, s, t);
+    mpz_clear(r);
   }
   mpz_init_set(sreduced, s);
-  mpz_init(t);
-  ret = 1;
+
+  ret = 0;
+  mpz_sub_ui(t, n, 1);
+  if (mpz_cmp(s,t) == 0 && mpz_kronecker(a,n) != -1)
+    goto DONE_IPR;
+  /* Unclear if this is worth doing.
+  i = is_power(a, 0);
+  if (i > 1 && mpz_gcd_ui(NULL, s, i) != 1)
+    goto DONE_IPR;
+  */
 
 #define IPR_TEST_UI(s, p, a, n, t, ret) \
   mpz_divexact_ui(t, s, p); \
@@ -187,6 +251,7 @@ int is_primitive_root(mpz_t ina, mpz_t n, int nprime)
   mpz_powm(t, a, t, n); \
   if (mpz_cmp_ui(t, 1) == 0) { ret = 0; }
 
+  ret = 1;
   { /* Pull out small factors and test */
     UV p, fromp = 0;
     while (ret == 1 && (p = _GMP_trial_factor(sreduced, fromp, 60))) {
@@ -996,6 +1061,21 @@ void mpf_pow(mpf_t powx, mpf_t b, mpf_t x)
   mpf_clear(t);
 }
 
+void mpf_root(mpf_t rootx, mpf_t x, mpf_t n)
+{
+  if (mpf_sgn(n) == 0) {
+   mpf_set_ui(rootx, 0);
+  } else if (mpf_cmp_ui(n, 2) == 0) {
+    mpf_sqrt(rootx, x);
+  } else {
+    mpf_t t;
+    mpf_init2(t, mpf_get_prec(rootx));
+    mpf_ui_div(t, 1, n);
+    mpf_pow(rootx, x, t);
+    mpf_clear(t);
+  }
+}
+
 void mpf_agm(mpf_t r, mpf_t a, mpf_t b)
 {
   mpf_t t;
@@ -1714,6 +1794,16 @@ void polyz_roots_modp(mpz_t** roots, long *nroots, long maxroots,
 
 #include "class_poly_data.h"
 
+const char* poly_class_type_name(int type)
+{
+  switch (type) {
+    case 1: return "Hilbert";
+    case 2: return "Weber";
+    case 3: return "Ramanujan";
+    default: return "Unknown";
+  }
+}
+
 int* poly_class_nums(void)
 {
   int* dlist;
